@@ -17,6 +17,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <locale.h>
+#include <setjmp.h>
 
 #define LKC_DIRECT_LINK
 #include "lkc.h"
@@ -719,6 +720,12 @@ static void conf_string(struct menu *menu)
 		case S_HEX:
 			heading = _(inputbox_instructions_hex);
 			break;
+		case S_EXECUTE:
+			execute_sym(menu->sym, 1);
+			touchwin(stdscr);
+			touchwin(curscr);
+			wrefresh(curscr);
+			return;
 		case S_STRING:
 			heading = _(inputbox_instructions_string);
 			break;
@@ -798,6 +805,105 @@ static void conf_save(void)
 	}
 }
 
+static jmp_buf buf;
+
+int m_do_run_cmds(char *cmd, int restart)
+{
+	char b[1024]; int res;
+	execute_sym_question(cmd, b, restart, 1);
+	res = dialog_yesno(NULL,
+			   b,
+			   20, 80);
+	return res == 0 ? 1 : 0;
+}
+
+int show_intro(void)
+{
+	int res;
+	const char *title = 0; const char *prompt = 0; int height = 20; int width = 80;
+	prompt = "Note: Commands can be executed from within the configuratior now\n"
+		"by double-clicking on \"Execute ...\" configuration entries.\n"
+		"It is not neccessary any more to exit the GUI for \ni.e. \"install/upgrade\" and \"build\".\n"
+		"The LINUX and BUILDROOT configuration trees are accessible\n"
+		"as subtrees in this configurator.\n";
+	
+	{
+		int i, x, y, key = 0, button = 0;
+		WINDOW *dialog;
+		
+	do_resize:
+		if (getmaxy(stdscr) < (height + 4))
+			return -ERRDISPLAYTOOSMALL;
+		if (getmaxx(stdscr) < (width + 4))
+			return -ERRDISPLAYTOOSMALL;
+		
+		/* center dialog box on screen */
+		x = (COLS - width) / 2;
+		y = (LINES - height) / 2;
+		
+		draw_shadow(stdscr, y, x, height, width);
+		
+		dialog = newwin(height, width, y, x);
+		keypad(dialog, TRUE);
+		
+		draw_box(dialog, 0, 0, height, width,
+			 dlg.dialog.atr, dlg.border.atr);
+		wattrset(dialog, dlg.border.atr);
+		mvwaddch(dialog, height - 3, 0, ACS_LTEE);
+		for (i = 0; i < width - 2; i++)
+			waddch(dialog, ACS_HLINE);
+		wattrset(dialog, dlg.dialog.atr);
+		waddch(dialog, ACS_RTEE);
+		
+		print_title(dialog, title, width);
+		
+		wattrset(dialog, dlg.dialog.atr);
+		print_autowrap(dialog, prompt, width - 2, 1, 3);
+
+		{
+			int x = width / 2;
+			int y = height - 2;
+			print_button(dialog, gettext(" Ok "), y, x, 1);
+		}
+		
+		while (key != KEY_ESC) {
+			key = wgetch(dialog);
+			switch (key) {
+			case 'Y':
+			case 'y':
+				delwin(dialog);
+				return 0;
+			case 'N':
+			case 'n':
+				delwin(dialog);
+				return 1;
+				
+			case ' ':
+			case '\n':
+				delwin(dialog);
+				return button;
+			case KEY_ESC:
+				key = on_key_esc(dialog);
+				break;
+			case KEY_RESIZE:
+				delwin(dialog);
+				on_key_resize();
+				goto do_resize;
+			}
+		}
+		
+		delwin(dialog);
+		return key;		/* ESC pressed */
+	}
+	return 0;
+}
+
+
+void m_restart_app(void) 
+{
+	longjmp(buf,1);
+}
+
 int main(int ac, char **av)
 {
 	int saved_x, saved_y;
@@ -826,19 +932,27 @@ int main(int ac, char **av)
 		return 1;
 	}
 
+	f_do_run_cmds = m_do_run_cmds;
+	f_restart_app = m_restart_app;
+	
 	set_config_filename(conf_get_configname());
-	do {
-		conf(&rootmenu);
-		dialog_clear();
-		if (conf_get_changed())
-			res = dialog_yesno(NULL,
-					   _("Do you wish to save your "
-					     "new configuration?\n"
-					     "<ESC><ESC> to continue."),
-					   6, 60);
-		else
-			res = -1;
-	} while (res == KEY_ESC);
+
+	show_intro();
+	
+	if ( ! setjmp(buf) ) {
+		do {
+			conf(&rootmenu);
+			dialog_clear();
+			if (conf_get_changed())
+				res = dialog_yesno(NULL,
+						   _("Do you wish to save your "
+						     "new configuration?\n"
+						     "<ESC><ESC> to continue."),
+						   6, 60);
+			else
+				res = -1;
+		} while (res == KEY_ESC);
+	}	
 	end_dialog(saved_x, saved_y);
 
 	switch (res) {
@@ -862,6 +976,6 @@ int main(int ac, char **av)
 			"\n\n"));
 	}
 
-	return 0;
+	return get_restart();
 }
 
